{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {},
   "outputs": [],
   "source": [
    "import pandas as pd\n",
    "import altair as alt\n",
    "from load_decks import load_decks, deck_card_colors"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "0\n",
      "25\n",
      "50\n",
      "75\n",
      "100\n",
      "125\n",
      "150\n",
      "175\n",
      "200\n",
      "225\n",
      "250\n",
      "275\n",
      "300\n",
      "325\n",
      "350\n",
      "375\n",
      "400\n",
      "425\n",
      "450\n",
      "475\n",
      "500\n",
      "525\n",
      "550\n",
      "575\n",
      "600\n",
      "625\n",
      "650\n",
      "675\n"
     ]
    }
   ],
   "source": [
    "raw_decks = load_decks()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {},
   "outputs": [],
   "source": [
    "decks = pd.DataFrame([deck_card_colors(d) for d in raw_decks if d['cards'] is not None]).set_index('id')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {},
   "outputs": [],
   "source": [
    "for c in 'WUBRG':\n",
    "    decks[c + 'd'] = decks[c] >= 5"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "166"
      ]
     },
     "execution_count": 5,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "decks['player'].nunique()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {},
   "outputs": [],
   "source": [
    "decks['primary_color'] = decks[list('WUBRG')].idxmax(axis=1)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {},
   "outputs": [],
   "source": [
    "def has_card(deck_id, card_name):\n",
    "    deck = next(deck for deck in raw_decks if deck['id'] == deck_id)\n",
    "    return any(card['card']['name'] == card_name for card in deck['cards'])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "metadata": {},
   "outputs": [],
   "source": [
    "decks['hoof?'] = decks.apply(lambda row: has_card(row.name, 'Craterhoof Behemoth'), axis=1)\n",
    "decks['shatter?'] = decks.apply(lambda row: has_card(row.name, 'Shatter the Sky'), axis=1)\n",
    "decks['cast?'] = decks.apply(lambda row: has_card(row.name, 'Cast Down'), axis=1)\n",
    "decks['ecd?'] = decks.apply(lambda row: has_card(row.name, 'Elspeth Conquers Death'), axis=1)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 9,
   "metadata": {},
   "outputs": [],
   "source": [
    "completed_decks = decks[(decks['match_wins'] + decks['match_losses']) == 5]"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 10,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "match_wins       416\n",
       "match_losses     416\n",
       "player           416\n",
       "resigned         416\n",
       "creation_date    416\n",
       "W                416\n",
       "U                416\n",
       "B                416\n",
       "R                416\n",
       "G                416\n",
       "Wd               416\n",
       "Ud               416\n",
       "Bd               416\n",
       "Rd               416\n",
       "Gd               416\n",
       "primary_color    416\n",
       "hoof?            416\n",
       "shatter?         416\n",
       "cast?            416\n",
       "ecd?             416\n",
       "dtype: int64"
      ]
     },
     "execution_count": 10,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "completed_decks.count()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 11,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "15.407407407407407"
      ]
     },
     "execution_count": 11,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "416 / (4 * 7 - 1)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 12,
   "metadata": {},
   "outputs": [],
   "source": [
    "matches = set((match['winner']['id'], match['loser']['id']) for deck in raw_decks for match in deck['matches'])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "metadata": {},
   "outputs": [],
   "source": [
    "def mtg_colors(name, color_names=list('WUBRG')):\n",
    "    return alt.Color(name, scale={'domain': color_names, 'range': ['#fffcda', '#ace2fa', '#ccc2c0', '#f8aa92', '#9dd2af']}, legend=None)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 116,
   "metadata": {},
   "outputs": [],
   "source": [
    "alt.Chart(completed_decks[['player', 'primary_color']]).mark_bar().encode(\n",
    "    x=alt.X('player:N', sort='-y', axis=None),\n",
    "    y=alt.Y('count():Q', title='Completed League Runs'),\n",
    "    color=mtg_colors('primary_color:N')\n",
    ").properties(width='container', title='Completed League Runs per Player (with Primary Color)').save('jmp_league_completions.json')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "13"
      ]
     },
     "execution_count": 15,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "(completed_decks.groupby('player').count()['resigned'] >= 10).sum()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 16,
   "metadata": {},
   "outputs": [],
   "source": [
    "deck_colors_weekly = completed_decks.groupby(pd.Grouper(key='creation_date', freq='7d'))[[c + 'd' for c in 'WUBRG']].apply(lambda group: group.sum() / group.count()).reset_index().melt(id_vars=['creation_date'])\n",
    "\n",
    "alt.Chart(deck_colors_weekly).mark_bar().encode(\n",
    "    x=alt.X('variable:N', axis=None),\n",
    "    y=alt.Y('value:Q', title='Fraction of Decks'),\n",
    "    color=mtg_colors('variable:N', color_names=['Wd', 'Ud', 'Bd', 'Rd', 'Gd']),\n",
    "    column=alt.Column('creation_date:O', title='Week Start', header=alt.Header(format='%d %b', formatType='time', orient='bottom', titlePadding=0))\n",
    ").properties(\n",
    "    width=155,\n",
    "    height=200,\n",
    "    title='Weekly Breakdown of Colors in All Completed Runs',\n",
    ").configure_title(anchor='middle').save('jmp_weekly_breakdown_completed.json')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "metadata": {},
   "outputs": [],
   "source": [
    "deck_colors_weekly = completed_decks[completed_decks['match_wins'] >= 3].groupby(pd.Grouper(key='creation_date', freq='7d'))[[c + 'd' for c in 'WUBRG']].apply(lambda group: group.sum() / group.count()).reset_index().melt(id_vars=['creation_date'])\n",
    "\n",
    "alt.Chart(deck_colors_weekly).mark_bar().encode(\n",
    "    x=alt.X('variable:N', axis=None),\n",
    "    y=alt.Y('value:Q', title='Fraction of Decks'),\n",
    "    color=mtg_colors('variable:N', color_names=['Wd', 'Ud', 'Bd', 'Rd', 'Gd']),\n",
    "    column=alt.Column('creation_date:O', title='Week Start', header=alt.Header(format='%d %b', formatType='time', orient='bottom', titlePadding=0))\n",
    ").properties(\n",
    "    width=155,\n",
    "    height=200,\n",
    "    title='Weekly Breakdown of Colors in w/ 3+ Wins',\n",
    ").configure_title(anchor='middle').save('jmp_weekly_breakdown_winning.json')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 18,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<div id=\"altair-viz-22f361dce5b84cb7bb39f8e7c00c3f88\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-22f361dce5b84cb7bb39f8e7c00c3f88\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-22f361dce5b84cb7bb39f8e7c00c3f88\");\n",
       "    }\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm//vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm//vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm//vega-lite@4.8.1?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm//vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function loadScript(lib) {\n",
       "      return new Promise(function(resolve, reject) {\n",
       "        var s = document.createElement('script');\n",
       "        s.src = paths[lib];\n",
       "        s.async = true;\n",
       "        s.onload = () => resolve(paths[lib]);\n",
       "        s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "        document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "      });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      require([\"vega-embed\"], displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else if (typeof vegaEmbed === \"function\") {\n",
       "      displayChart(vegaEmbed);\n",
       "    } else {\n",
       "      loadScript(\"vega\")\n",
       "        .then(() => loadScript(\"vega-lite\"))\n",
       "        .then(() => loadScript(\"vega-embed\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 400, \"continuousHeight\": 300}, \"title\": {\"anchor\": \"middle\"}}, \"data\": {\"name\": \"data-a75799239e66607a697919460210c7ab\"}, \"mark\": \"bar\", \"encoding\": {\"column\": {\"type\": \"ordinal\", \"field\": \"creation_date\", \"header\": {\"format\": \"%d %b\", \"formatType\": \"time\", \"orient\": \"bottom\", \"titlePadding\": 0}, \"title\": \"Week Start\"}, \"x\": {\"type\": \"nominal\", \"field\": \"variable\", \"title\": null}, \"y\": {\"type\": \"quantitative\", \"field\": \"value\", \"title\": \"Fraction of Decks\"}}, \"height\": 200, \"title\": \"Weekly Breakdown of Hoof and Removal\", \"width\": 155, \"$schema\": \"https://vega.github.io/schema/vega-lite/v4.8.1.json\", \"datasets\": {\"data-a75799239e66607a697919460210c7ab\": [{\"creation_date\": \"2020-07-16T00:00:00+00:00\", \"variable\": \"Elspeth Conquers Death\", \"value\": 0.22535211267605634}, {\"creation_date\": \"2020-07-23T00:00:00+00:00\", \"variable\": \"Elspeth Conquers Death\", \"value\": 0.19298245614035087}, {\"creation_date\": \"2020-07-30T00:00:00+00:00\", \"variable\": \"Elspeth Conquers Death\", \"value\": 0.3829787234042553}, {\"creation_date\": \"2020-08-06T00:00:00+00:00\", \"variable\": \"Elspeth Conquers Death\", \"value\": 0.3076923076923077}, {\"creation_date\": \"2020-07-16T00:00:00+00:00\", \"variable\": \"Craterhoof Behemoth\", \"value\": 0.2746478873239437}, {\"creation_date\": \"2020-07-23T00:00:00+00:00\", \"variable\": \"Craterhoof Behemoth\", \"value\": 0.2631578947368421}, {\"creation_date\": \"2020-07-30T00:00:00+00:00\", \"variable\": \"Craterhoof Behemoth\", \"value\": 0.1702127659574468}, {\"creation_date\": \"2020-08-06T00:00:00+00:00\", \"variable\": \"Craterhoof Behemoth\", \"value\": 0.15384615384615385}, {\"creation_date\": \"2020-07-16T00:00:00+00:00\", \"variable\": \"Shatter the Sky\", \"value\": 0.19718309859154928}, {\"creation_date\": \"2020-07-23T00:00:00+00:00\", \"variable\": \"Shatter the Sky\", \"value\": 0.2807017543859649}, {\"creation_date\": \"2020-07-30T00:00:00+00:00\", \"variable\": \"Shatter the Sky\", \"value\": 0.19148936170212766}, {\"creation_date\": \"2020-08-06T00:00:00+00:00\", \"variable\": \"Shatter the Sky\", \"value\": 0.23076923076923078}, {\"creation_date\": \"2020-07-16T00:00:00+00:00\", \"variable\": \"Cast Down\", \"value\": 0.13380281690140844}, {\"creation_date\": \"2020-07-23T00:00:00+00:00\", \"variable\": \"Cast Down\", \"value\": 0.22807017543859648}, {\"creation_date\": \"2020-07-30T00:00:00+00:00\", \"variable\": \"Cast Down\", \"value\": 0.2127659574468085}, {\"creation_date\": \"2020-08-06T00:00:00+00:00\", \"variable\": \"Cast Down\", \"value\": 0.15384615384615385}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.Chart(...)"
      ]
     },
     "execution_count": 18,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "deck_cards_weekly = completed_decks[completed_decks['match_wins'] >= 3].groupby(pd.Grouper(key='creation_date', freq='7d'))[['ecd?', 'hoof?', 'shatter?', 'cast?']].apply(lambda group: group.sum() / group.count()).reset_index().melt(id_vars=['creation_date'])\n",
    "\n",
    "alt.Chart(deck_cards_weekly.replace({\n",
    "    'ecd?': \"Elspeth Conquers Death\",\n",
    "    'hoof?': \"Craterhoof Behemoth\",\n",
    "    'cast?': \"Cast Down\",\n",
    "    'shatter?': \"Shatter the Sky\"\n",
    "})).mark_bar().encode(\n",
    "    x=alt.X('variable:N', title=None),\n",
    "    y=alt.Y('value:Q', title='Fraction of Decks'),\n",
    "    column=alt.Column('creation_date:O', title='Week Start', header=alt.Header(format='%d %b', formatType='time', orient='bottom', titlePadding=0))\n",
    ").properties(\n",
    "    width=155,\n",
    "    height=200,\n",
    "    title='Weekly Breakdown of Hoof and Removal',\n",
    ").configure_title(anchor='middle')#.save('jmp_weekly_breakdown_removal.json')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 91,
   "metadata": {},
   "outputs": [],
   "source": [
    "matches = set((match['winner']['id'], match['loser']['id'], match['date']) for deck in raw_decks for match in deck['matches'])\n",
    "\n",
    "matches = pd.DataFrame(matches, columns=['winner', 'loser', 'date'])\n",
    "matches['date'] = pd.to_datetime(matches['date'])"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 93,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "winner                  int64\n",
       "loser                   int64\n",
       "date      datetime64[ns, UTC]\n",
       "dtype: object"
      ]
     },
     "execution_count": 93,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "matches.dtypes"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 103,
   "metadata": {},
   "outputs": [],
   "source": [
    "alt.Chart(matches).mark_bar(tooltip=True).encode(\n",
    "    x=alt.X('date:T', timeUnit='hours', title=\"Time of Day (Local Time)\"),\n",
    "    y=alt.Y('count()', title=\"# of Matches\")\n",
    ").properties(\n",
    "    width='container',\n",
    "    title='League Match Play Times'\n",
    ").save('jmp_play_hours.json')"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 112,
   "metadata": {},
   "outputs": [],
   "source": [
    "alt.Chart(matches).mark_bar(tooltip=True).encode(\n",
    "    x=alt.X('monthdate(date)', title=\"Date\"),\n",
    "    y=alt.Y('count()', title=\"# of Matches Played\"),\n",
    ").properties(\n",
    "    width='container',\n",
    "    title='League Match Play Dates'\n",
    ").save('jmp_play_dates.json')"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.7.1"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
